<?php

namespace Highwaypro\App\Data\Model\Preferences;

use HighWayPro\Original\Collections\Collection;
use HighwayPro\Original\Collections\JSONMapper;
use HighwayPro\Original\Collections\Mapper\Mappable;
use HighwayPro\Original\Collections\Mapper\Types;
use Highwaypro\App\Data\Model\Preferences\Preferences;

Class UrlPreferences extends Preferences
{
    const CHARACTER_ALPHABETIC   = 'alphabetical';
    const CHARACTER_NUMERIC      = 'numeric';
    const CHARACTER_ALPHANUMERIC = 'alphanumeric';

    const CASE_ANY       = 'any';
    const CASE_UPPERCASE = 'uppercase';
    const CASE_LOWERCASE = 'lowercase';


    static public function fields()
    {
        return new Collection([
        'generateUrlsOnNewPosts_isEnabled'  => Types::BOOLEAN()->withDefault(true),
        'postUrl_UrlType'                   => Types::INTEGER(), // the base path of post urls
        'autoGeneratedPathType'             => Types::STRING()->withDefault(self::CHARACTER_NUMERIC) 
                                               ->allowed([
                                                    self::CHARACTER_ALPHABETIC, 
                                                    self::CHARACTER_NUMERIC, 
                                                    self::CHARACTER_ALPHANUMERIC
                                                ]),
        'autoGeneratedPathAlphanumericCase' => Types::STRING()->withDefault(self::CASE_ANY) 
                                               ->allowed([
                                                    self::CASE_ANY, 
                                                    self::CASE_UPPERCASE, 
                                                    self::CASE_LOWERCASE
                                                ]),
        'autoGeneratedPathLength'           => Types::INTEGER()->withDefault(6)->minimum(2), 
        ]);   
    }

    protected function getMap()
    {
        return static::fields()->asArray();
    }

    public function __construct($data)
    {
        $this->preferences = $this->map($data);
    }
}